@@ -1,43 +1,17 @@
 """
-This module provides 3 different methods to perform 'all relevant feature selection'
-
-
-Reference:
-----------
-NILSSON, Roland, PEÑA, José M., BJÖRKEGREN, Johan, et al.
-Consistent feature selection for pattern recognition in polynomial time.
-Journal of Machine Learning Research, 2007, vol. 8, no Mar, p. 589-612.
-
-KURSA, Miron B., RUDNICKI, Witold R., et al.
-Feature selection with the Boruta package.
-J Stat Softw, 2010, vol. 36, no 11, p. 1-13.
-
-https://github.com/chasedehan/BoostARoota
-
-The module structure is the following:
----------------------------------------
-- The ``Leshy`` class, a heavy re-work of ``BorutaPy`` class
-  itself a modified version of Boruta, the pull request I submitted and still pending:
-  https://github.com/scikit-learn-contrib/boruta_py/pull/77
-
-- The ``BoostAGroota`` class, a modified version of BoostARoota, PR still to be submitted
-  https://github.com/chasedehan/BoostARoota
-
-- The ``GrootCV`` class for a new method for all relevant feature selection using a lgGBM model,
-  cross-validated SHAP importances and shadowing.
+Utility and validation functions
+  
 """
 
 from __future__ import print_function, division
 
 import lightgbm as lgb
 import matplotlib as mpl
-import seaborn as sns
 import itertools
 import numpy as np
 import pandas as pd
 
 from pkg_resources import resource_filename
-from numpy.random import choice
 from matplotlib import pyplot as plt
 from sklearn.model_selection import train_test_split
 from sklearn.inspection import permutation_importance
@@ -58,6 +32,7 @@ from palettable.cartocolors.qualitative import Bold_10
 #                   #
 #####################
 
+
 def check_if_tree_based(model):
     """check if estimator is tree based
 
@@ -71,10 +46,11 @@ def check_if_tree_based(model):
     condition : boolean
         if tree based or not
     """
-    tree_based_models = ['lightgbm', 'xgboost', 'catboost', '_forest', 'boosting']
+    tree_based_models = ["lightgbm", "xgboost", "catboost", "_forest", "boosting"]
     condition = any(i in str(type(model)).lower() for i in tree_based_models)
     return condition
 
+
 def is_lightgbm(estimator):
     """check if estimator is lightgbm
 
@@ -88,7 +64,7 @@ def is_lightgbm(estimator):
     condition : boolean
         if lgbm based or not
     """
-    is_lgb = 'lightgbm' in str(type(estimator))
+    is_lgb = "lightgbm" in str(type(estimator))
     return is_lgb
 
 
@@ -105,7 +81,7 @@ def is_catboost(estimator):
     condition : boolean
         if catboost based or not
     """
-    is_cat = 'catboost' in str(type(estimator))
+    is_cat = "catboost" in str(type(estimator))
     return is_cat
 
 
@@ -122,10 +98,10 @@ def is_xgboost(estimator):
     condition : boolean
         if xgboost based or not
     """
-    is_xgb = 'xgboost' in str(type(estimator))
+    is_xgb = "xgboost" in str(type(estimator))
     return is_xgb
 
-    
+
 def LightForestRegressor(n_feat, n_estimators=10):
     """lightGBM implementation of the Random Forest regressor with the
     ideal number of features, according to Elements of statistical learning
@@ -144,8 +120,15 @@ def LightForestRegressor(n_feat, n_estimators=10):
     """
 
     feat_frac = n_feat / (3 * n_feat)
-    return lgb.LGBMRegressor(verbose=-1, force_col_wise=True, n_estimators=n_estimators, subsample=0.632,
-                             colsample_bytree=feat_frac, boosting_type="rf", subsample_freq=1)
+    return lgb.LGBMRegressor(
+        verbose=-1,
+        force_col_wise=True,
+        n_estimators=n_estimators,
+        subsample=0.632,
+        colsample_bytree=feat_frac,
+        boosting_type="rf",
+        subsample_freq=1,
+    )
 
 
 def LightForestClassifier(n_feat, n_estimators=10):
@@ -165,11 +148,17 @@ def LightForestClassifier(n_feat, n_estimators=10):
         sklearn random forest estimator based on lightgbm
     """
     feat_frac = np.sqrt(n_feat) / n_feat
-    return lgb.LGBMClassifier(verbose=-1, force_col_wise=True, n_estimators=n_estimators, subsample=0.632,
-                              colsample_bytree=feat_frac, boosting_type="rf", subsample_freq=1)
-    
-    
-    
+    return lgb.LGBMClassifier(
+        verbose=-1,
+        force_col_wise=True,
+        n_estimators=n_estimators,
+        subsample=0.632,
+        colsample_bytree=feat_frac,
+        boosting_type="rf",
+        subsample_freq=1,
+    )
+
+
 def is_list_of_str(str_list):
     """Check if ``str_list`` is not a list of strings
 
@@ -184,11 +173,14 @@ def is_list_of_str(str_list):
         True if list of strings, else False
     """
     if str_list is not None:
-        if not (isinstance(str_list, list) and all(isinstance(s, str) for s in str_list)):
+        if not (
+            isinstance(str_list, list) and all(isinstance(s, str) for s in str_list)
+        ):
             return False
         else:
             return True
 
+
 def is_list_of_bool(bool_list):
     """Check if ``bool_list`` is not a list of Booleans
 
@@ -203,11 +195,14 @@ def is_list_of_bool(bool_list):
         True if list of Booleans, else False
     """
     if bool_list is not None:
-        if not (isinstance(bool_list, list) and all(isinstance(s, bool) for s in bool_list)):
+        if not (
+            isinstance(bool_list, list) and all(isinstance(s, bool) for s in bool_list)
+        ):
             return False
         else:
             return True
-        
+
+
 def is_list_of_int(int_list):
     """Check if ``int_list`` is not a list of integers
 
@@ -222,11 +217,13 @@ def is_list_of_int(int_list):
         True if list of integers, else False
     """
     if int_list is not None:
-        if not (isinstance(int_list, list) and all(isinstance(s, int) for s in int_list)):
+        if not (
+            isinstance(int_list, list) and all(isinstance(s, int) for s in int_list)
+        ):
             return False
         else:
             return True
-        
+
 
 def set_my_plt_style(height=3, width=5, linewidth=2):
     """This set the style of matplotlib to fivethirtyeight with some modifications (colours, axes)
@@ -241,21 +238,27 @@ def set_my_plt_style(height=3, width=5, linewidth=2):
         fig width in inches, by default 2
     """
 
-    plt.style.use('fivethirtyeight')
+    plt.style.use("fivethirtyeight")
     my_colors_list = Bold_10.hex_colors
     myorder = [2, 3, 4, 1, 0, 6, 5, 8, 9, 7]
     my_colors_list = [my_colors_list[i] for i in myorder]
     bckgnd_color = "#f5f5f5"
-    params = {'figure.figsize': (width, height), "axes.prop_cycle": plt.cycler(color=my_colors_list),
-              "axes.facecolor": bckgnd_color, "patch.edgecolor": bckgnd_color,
-              "figure.facecolor": bckgnd_color,
-              "axes.edgecolor": bckgnd_color, "savefig.edgecolor": bckgnd_color,
-              "savefig.facecolor": bckgnd_color, "grid.color": "#d2d2d2",
-              'lines.linewidth': linewidth}  # plt.cycler(color=my_colors_list)
+    params = {
+        "figure.figsize": (width, height),
+        "axes.prop_cycle": plt.cycler(color=my_colors_list),
+        "axes.facecolor": bckgnd_color,
+        "patch.edgecolor": bckgnd_color,
+        "figure.facecolor": bckgnd_color,
+        "axes.edgecolor": bckgnd_color,
+        "savefig.edgecolor": bckgnd_color,
+        "savefig.facecolor": bckgnd_color,
+        "grid.color": "#d2d2d2",
+        "lines.linewidth": linewidth,
+    }  # plt.cycler(color=my_colors_list)
     mpl.rcParams.update(params)
 
 
-def highlight_tick(str_match, figure, color='red', axis='y'):
+def highlight_tick(str_match, figure, color="red", axis="y"):
     """Highlight the x/y tick-labels if they contains a given string
 
     Parameters
@@ -280,11 +283,11 @@ def highlight_tick(str_match, figure, color='red', axis='y'):
         if axis is not 'x' or 'y'
     """
 
-    if axis == 'y':
+    if axis == "y":
         labels = [item.get_text() for item in figure.gca().get_yticklabels()]
         indices = [i for i, s in enumerate(labels) if str_match in s]
         [figure.gca().get_yticklabels()[idx].set_color(color) for idx in indices]
-    elif axis == 'x':
+    elif axis == "x":
         labels = [item.get_text() for item in figure.gca().get_xticklabels()]
         indices = [i for i, s in enumerate(labels) if str_match in s]
         [figure.gca().get_xticklabels()[idx].set_color(color) for idx in indices]
@@ -294,7 +297,7 @@ def highlight_tick(str_match, figure, color='red', axis='y'):
     return figure
 
 
-def sklearn_pimp_bench(model, X, y, task='regression', sample_weight=None):
+def sklearn_pimp_bench(model, X, y, task="regression", sample_weight=None):
     """Benchmark using sklearn permutation importance, works for regression and classification.
 
     Parameters
@@ -329,35 +332,47 @@ def sklearn_pimp_bench(model, X, y, task='regression', sample_weight=None):
     # your-tree-based-ensembles-worse-heres-why-d64b282b5769
     X, cat_var_df, inv_mapper = cat_var(X)
 
-    if task == 'regression':
+    if task == "regression":
         stratify = None
-    elif task == 'classification':
+    elif task == "classification":
         stratify = y
     else:
         raise ValueError("`task` should be either 'regression' or 'classification' ")
 
     if sample_weight is not None:
-        X_train, X_test, y_train, y_test, w_train, w_test = \
-            train_test_split(X, y, sample_weight, stratify=stratify, random_state=42)
+        X_train, X_test, y_train, y_test, w_train, w_test = train_test_split(
+            X, y, sample_weight, stratify=stratify, random_state=42
+        )
     else:
-        X_train, X_test, y_train, y_test = train_test_split(X, y, stratify=stratify, random_state=42)
+        X_train, X_test, y_train, y_test = train_test_split(
+            X, y, stratify=stratify, random_state=42
+        )
         w_train, w_test = None, None
 
     # lightgbm faster and better than RF
 
     model.fit(X_train, y_train, sample_weight=w_train)
-    result = permutation_importance(model, X_test, y_test, n_repeats=10,
-                                    random_state=42, n_jobs=2, sample_weight=w_test)
+    result = permutation_importance(
+        model,
+        X_test,
+        y_test,
+        n_repeats=10,
+        random_state=42,
+        n_jobs=2,
+        sample_weight=w_test,
+    )
 
     sorted_idx = result.importances_mean.argsort()
     # Plot (5 predictors per inch)
     fig, ax = plt.subplots(figsize=(16, X.shape[1] / 5))
-    ax.boxplot(result.importances[sorted_idx].T, vert=False, labels=X_test.columns[sorted_idx])
+    ax.boxplot(
+        result.importances[sorted_idx].T, vert=False, labels=X_test.columns[sorted_idx]
+    )
     ax.set_title("Permutation Importances (test set)")
     fig.tight_layout()
-    indices = [i for i, s in enumerate(X_test.columns[sorted_idx]) if 'random' in s]
+    indices = [i for i, s in enumerate(X_test.columns[sorted_idx]) if "random" in s]
     [fig.gca().get_yticklabels()[idx].set_color("red") for idx in indices]
-    indices = [i for i, s in enumerate(X_test.columns[sorted_idx]) if 'genuine' in s]
+    indices = [i for i, s in enumerate(X_test.columns[sorted_idx]) if "genuine" in s]
     [fig.gca().get_yticklabels()[idx].set_color("green") for idx in indices]
     plt.show()
     return fig
@@ -380,10 +395,17 @@ def compare_varimp(feat_selector, models, X, y, sample_weight=None):
         sample weights if any, by default None
     """
 
-    varimp_list = ['shap', 'pimp', 'native']
+    varimp_list = ["shap", "pimp", "native"]
     for model, varimp in itertools.product(models, varimp_list):
-        print('=' * 20 + ' ' + str(feat_selector.__class__.__name__) +
-              ' - testing: {mod:>25} for var.imp: {vimp:<15} '.format(mod=str(model.__class__.__name__), vimp=varimp) + '=' * 20)
+        print(
+            "=" * 20
+            + " "
+            + str(feat_selector.__class__.__name__)
+            + " - testing: {mod:>25} for var.imp: {vimp:<15} ".format(
+                mod=str(model.__class__.__name__), vimp=varimp
+            )
+            + "=" * 20
+        )
         # change the varimp
         feat_selector.importance = varimp
         # change model
@@ -397,24 +419,11 @@ def compare_varimp(feat_selector, models, X, y, sample_weight=None):
 
         if fig is not None:
             # highlight synthetic random variable
-            fig = highlight_tick(figure=fig, str_match='random')
-            fig = highlight_tick(figure=fig, str_match='genuine', color='green')
+            fig = highlight_tick(figure=fig, str_match="random")
+            fig = highlight_tick(figure=fig, str_match="genuine", color="green")
             plt.show()
 
 
-    """Identify categorical features.
-
-        Parameters
-        ----------
-        df: original df after missing operations
-
-        Returns
-        -------
-        cat_var_df: summary df with col index and col name for all categorical vars
-        :param return_cat: Boolean, return encoded cols as type 'category'
-        :param df: pd.DF, the encoded data-frame
-        :param col_excl: list, colums not to be encoded
-        """
 def cat_var(df, col_excl=None, return_cat=True):
     """Encode categorical variables using integers
 
@@ -434,25 +443,34 @@ def cat_var(df, col_excl=None, return_cat=True):
     """
 
     if col_excl is None:
-        non_num_cols = list(set(list(df.columns)) - set(list(df.select_dtypes(include=[np.number]))))
+        non_num_cols = list(
+            set(list(df.columns)) - set(list(df.select_dtypes(include=[np.number])))
+        )
     else:
         non_num_cols = list(
-            set(list(df.columns)) - set(list(df.select_dtypes(include=[np.number]))) - set(col_excl))
+            set(list(df.columns))
+            - set(list(df.select_dtypes(include=[np.number])))
+            - set(col_excl)
+        )
 
     cat_var_index = [df.columns.get_loc(c) for c in non_num_cols if c in df]
 
-    cat_var_df = pd.DataFrame({'cat_ind': cat_var_index,
-                               'cat_name': non_num_cols})
+    cat_var_df = pd.DataFrame({"cat_ind": cat_var_index, "cat_name": non_num_cols})
 
     cols_need_mapped = cat_var_df.cat_name.to_list()
-    inv_mapper = {col: dict(enumerate(df[col].astype('category').cat.categories)) for col in df[cols_need_mapped]}
-    mapper = {col: {v: k for k, v in inv_mapper[col].items()} for col in df[cols_need_mapped]}
+    inv_mapper = {
+        col: dict(enumerate(df[col].astype("category").cat.categories))
+        for col in df[cols_need_mapped]
+    }
+    mapper = {
+        col: {v: k for k, v in inv_mapper[col].items()} for col in df[cols_need_mapped]
+    }
 
     for c in cols_need_mapped:
         df.loc[:, c] = df.loc[:, c].map(mapper[c]).fillna(0).astype(int)
 
     if return_cat:
-        df[non_num_cols] = df[non_num_cols].astype('category')
+        df[non_num_cols] = df[non_num_cols].astype("category")
     return df, cat_var_df, inv_mapper
 
 
@@ -470,28 +488,43 @@ def _get_titanic_data():
     # plot_permutation_importance.html#sphx-glr-auto-examples-inspection-plot-permutation-importance-py
     X, y = fetch_openml("titanic", version=1, as_frame=True, return_X_y=True)
     rng = np.random.RandomState(seed=42)
-    nice_guys = ['Rick', 'Bender', 'Cartman', 'Morty', 'Fry', 'Vador', 'Thanos']
-    X['random_cat'] = np.random.choice(nice_guys, X.shape[0])
-    X['random_num'] = rng.randn(X.shape[0])
-    X['family_size'] = X['parch'] + X['sibsp']
-    X.drop(['parch', 'sibsp'], axis=1, inplace=True)
-    X['is_alone'] = 1
-    X.loc[X['family_size'] > 1, 'is_alone'] = 0
-    X['title'] = X['name'].str.split(", ", expand=True)[1].str.split(".", expand=True)[0]
-    X.loc[X.title == 'Miss', 'title'] = 'Mrs'
-    title_counts = X['title'].value_counts()
-    rare_titles = title_counts[title_counts<10].index
-    X.loc[X['title'].isin(rare_titles), 'title'] = 'rare'
+    nice_guys = ["Rick", "Bender", "Cartman", "Morty", "Fry", "Vador", "Thanos"]
+    X["random_cat"] = np.random.choice(nice_guys, X.shape[0])
+    X["random_num"] = rng.randn(X.shape[0])
+    X["family_size"] = X["parch"] + X["sibsp"]
+    X.drop(["parch", "sibsp"], axis=1, inplace=True)
+    X["is_alone"] = 1
+    X.loc[X["family_size"] > 1, "is_alone"] = 0
+    X["title"] = (
+        X["name"].str.split(", ", expand=True)[1].str.split(".", expand=True)[0]
+    )
+    X.loc[X.title == "Miss", "title"] = "Mrs"
+    title_counts = X["title"].value_counts()
+    rare_titles = title_counts[title_counts < 10].index
+    X.loc[X["title"].isin(rare_titles), "title"] = "rare"
     # X['title'] = X.title.apply(lambda x: 'rare' if rare_titles[x] else x)
 
-    categorical_columns = ['pclass', 'sex', 'embarked', 'random_cat', 'is_alone', 'title']
-    numerical_columns = ['age', 'family_size', 'fare', 'random_num']
+    categorical_columns = [
+        "pclass",
+        "sex",
+        "embarked",
+        "random_cat",
+        "is_alone",
+        "title",
+    ]
+    numerical_columns = ["age", "family_size", "fare", "random_num"]
     X = X[categorical_columns + numerical_columns]
     # Impute
-    categorical_pipe = Pipeline([('imputer', SimpleImputer(strategy='constant', fill_value='missing'))])
-    numerical_pipe = Pipeline([('imputer', SimpleImputer(strategy='mean'))])
+    categorical_pipe = Pipeline(
+        [("imputer", SimpleImputer(strategy="constant", fill_value="missing"))]
+    )
+    numerical_pipe = Pipeline([("imputer", SimpleImputer(strategy="mean"))])
     preprocessing = ColumnTransformer(
-        [('cat', categorical_pipe, categorical_columns), ('num', numerical_pipe, numerical_columns)])
+        [
+            ("cat", categorical_pipe, categorical_columns),
+            ("num", numerical_pipe, numerical_columns),
+        ]
+    )
     X_trans = preprocessing.fit_transform(X)
     X = pd.DataFrame(X_trans, columns=X.columns)
     # encode
@@ -500,10 +533,9 @@ def _get_titanic_data():
     X[numerical_columns] = X[numerical_columns].astype(float)
     # sample weight is just a dummy random vector for testing purpose
     sample_weight = np.random.uniform(0, 1, len(y))
-    return Bunch(data=X,
-                 target=y,
-                 sample_weight=sample_weight,
-                 categorical=categorical_columns)
+    return Bunch(
+        data=X, target=y, sample_weight=sample_weight, categorical=categorical_columns
+    )
 
 
 def _get_cancer_data():
@@ -521,15 +553,14 @@ def _get_cancer_data():
     X, y = data.data, data.target
     X = pd.DataFrame(X)
     X.columns = data.feature_names
-    X['random_num1'] = rng.randn(X.shape[0])
-    X['random_num2'] = np.random.poisson(1, X.shape[0])
+    X["random_num1"] = rng.randn(X.shape[0])
+    X["random_num2"] = np.random.poisson(1, X.shape[0])
     z = y.astype(int)
-    X['genuine_num'] = z * np.abs(np.random.normal(0, .1, X.shape[0])) + np.random.normal(0, 0.1, X.shape[0])
+    X["genuine_num"] = z * np.abs(
+        np.random.normal(0, 0.1, X.shape[0])
+    ) + np.random.normal(0, 0.1, X.shape[0])
     y = pd.Series(y)
-    return Bunch(data=X,
-                 target=y,
-                 sample_weight=None,
-                 categorical=None)
+    return Bunch(data=X, target=y, sample_weight=None, categorical=None)
 
 
 def _get_boston_data():
@@ -546,32 +577,59 @@ def _get_boston_data():
     rng = np.random.RandomState(seed=42)
     X = pd.DataFrame(boston.data)
     X.columns = boston.feature_names
-    X['random_num1'] = rng.randn(X.shape[0])
-    X['random_num2'] = np.random.poisson(1, X.shape[0])
+    X["random_num1"] = rng.randn(X.shape[0])
+    X["random_num2"] = np.random.poisson(1, X.shape[0])
     # high cardinality
-    X['random_cat'] = rng.randint(10 * X.shape[0], size=X.shape[0])
-    X['random_cat'] = 'cat_' + X['random_cat'].astype('str')
+    X["random_cat"] = rng.randint(10 * X.shape[0], size=X.shape[0])
+    X["random_cat"] = "cat_" + X["random_cat"].astype("str")
     # low cardinality
-    nice_guys = ['Rick', 'Bender', 'Cartman', 'Morty', 'Fry', 'Vador',
-                 'Thanos', 'Bejita', 'Cell', 'Tinkywinky', 'Lecter',
-                 'Alien', 'Terminator', 'Drago', 'Dracula',
-                 'Krueger', 'Geoffrey', 'Goldfinder', 'Blackbeard',
-                 'Excel', 'SAS', 'Bias', 'Variance', 'Scrum',
-                 'Human', 'Garry', 'Coldplay', 'Imaginedragons',
-                 'Platist', 'Creationist', 'Gruber', 'KeyserSoze', 'Luthor',
-                 'Klaue', 'Bane', 'MarkZ']
-    X['random_cat_2'] = np.random.choice(nice_guys, X.shape[0])
+    nice_guys = [
+        "Rick",
+        "Bender",
+        "Cartman",
+        "Morty",
+        "Fry",
+        "Vador",
+        "Thanos",
+        "Bejita",
+        "Cell",
+        "Tinkywinky",
+        "Lecter",
+        "Alien",
+        "Terminator",
+        "Drago",
+        "Dracula",
+        "Krueger",
+        "Geoffrey",
+        "Goldfinder",
+        "Blackbeard",
+        "Excel",
+        "SAS",
+        "Bias",
+        "Variance",
+        "Scrum",
+        "Human",
+        "Garry",
+        "Coldplay",
+        "Imaginedragons",
+        "Platist",
+        "Creationist",
+        "Gruber",
+        "KeyserSoze",
+        "Luthor",
+        "Klaue",
+        "Bane",
+        "MarkZ",
+    ]
+    X["random_cat_2"] = np.random.choice(nice_guys, X.shape[0])
     y = pd.Series(boston.target)
     # non linear noisy but genuine predictor to test the ability to detect even genuine noisy non-linearities
-    X['genuine_num'] = np.sqrt(y) + np.random.gamma(2, .5, X.shape[0])
-    cat_f = ['CHAS', 'RAD', 'random_cat', 'random_cat_2']
-    X[cat_f] = X[cat_f].astype(str).astype('category')
-    return Bunch(data=X,
-                 target=y,
-                 sample_weight=None,
-                 categorical=cat_f)
-    
-    
+    X["genuine_num"] = np.sqrt(y) + np.random.gamma(2, 0.5, X.shape[0])
+    cat_f = ["CHAS", "RAD", "random_cat", "random_cat_2"]
+    X[cat_f] = X[cat_f].astype(str).astype("category")
+    return Bunch(data=X, target=y, sample_weight=None, categorical=cat_f)
+
+
 def _load_housing(as_frame: bool = False):
     """Load the California housing data. See here
     https://scikit-learn.org/stable/modules/generated/sklearn.datasets.fetch_california_housing.html
@@ -581,30 +639,40 @@ def _load_housing(as_frame: bool = False):
     ----------
     as_frame :
         return a pandas dataframe? if not then a "Bunch" (enhanced dictionary) is returned (default ``True``)
-    
+
     Returns
     -------
     pd.DataFrame or Bunch
         the dataset
 
     """
-    fdescr_name = resource_filename(__name__, 'dataset/descr/housing.rst')
+    fdescr_name = resource_filename(__name__, "dataset/descr/housing.rst")
     with open(fdescr_name) as f:
         descr_text = f.read()
 
-    data_file_name = resource_filename(__name__, 'dataset/data/housing.zip')
+    data_file_name = resource_filename(__name__, "dataset/data/housing.zip")
     data = pd.read_csv(data_file_name)
-    feature_names = ['MedInc', 'HouseAge', 'AveRooms', 'AveBedrms',
-                     'Population', 'AveOccup', 'Latitude', 'Longitude']
+    feature_names = [
+        "MedInc",
+        "HouseAge",
+        "AveRooms",
+        "AveBedrms",
+        "Population",
+        "AveOccup",
+        "Latitude",
+        "Longitude",
+    ]
 
     if as_frame:
         return data
     else:
-        return Bunch(data=data[feature_names].values,
-                     target=data['target'].values,
-                     feature_names=feature_names,
-                     DESCR=descr_text,
-                     filename=data_file_name)
+        return Bunch(
+            data=data[feature_names].values,
+            target=data["target"].values,
+            feature_names=feature_names,
+            DESCR=descr_text,
+            filename=data_file_name,
+        )
 
 
 def plot_y_vs_X(X, y, ncols=2, figsize=(10, 10)):
@@ -664,8 +732,8 @@ def plot_y_vs_X(X, y, ncols=2, figsize=(10, 10)):
     return f
 
 
-def load_data(name='Titanic'):
-    """    Load some toy data set to test the All Relevant Feature Selection methods.
+def load_data(name="Titanic"):
+    """Load some toy data set to test the All Relevant Feature Selection methods.
     Dummies (random) predictors are added and ARFS should be able to filter them out.
     The Titanic predictors are encoded (needed for scikit estimators).
 
@@ -692,16 +760,18 @@ def load_data(name='Titanic'):
         if the dataset name is invalid
     """
 
-    if name == 'Titanic':
+    if name == "Titanic":
         return _get_titanic_data()
-    elif name == 'Boston':
+    elif name == "Boston":
         return _get_boston_data()
-    elif name == 'cancer':
+    elif name == "cancer":
         return _get_cancer_data()
-    elif name =="housing":
+    elif name == "housing":
         return _load_housing(as_frame=False)
     else:
-        raise ValueError("`name should be in ['Titanic', 'Boston', 'cancer', 'housing']`")
+        raise ValueError(
+            "`name should be in ['Titanic', 'Boston', 'cancer', 'housing']`"
+        )
 
 
 def _generated_corr_dataset_regr(size=1000):
@@ -729,10 +799,12 @@ def _generated_corr_dataset_regr(size=1000):
 
     # 5 relevant features, with positive and negative correlation to the target
     X[:, 0] = z
-    X[:, 1] = y * np.abs(np.random.normal(0, sigma * 2, size)) + np.random.normal(0, sigma / 10, size)
+    X[:, 1] = y * np.abs(np.random.normal(0, sigma * 2, size)) + np.random.normal(
+        0, sigma / 10, size
+    )
     X[:, 2] = -y + np.random.normal(0, sigma, size)
-    X[:, 3] = y ** 2 + np.random.normal(0, sigma, size)
-    X[:, 4] = np.sqrt(y) + np.random.gamma(1, .2, size)
+    X[:, 3] = y**2 + np.random.normal(0, sigma, size)
+    X[:, 4] = np.sqrt(y) + np.random.gamma(1, 0.2, size)
 
     # 5 irrelevant features
     X[:, 5] = np.random.normal(0, 1, size)
@@ -746,25 +818,55 @@ def _generated_corr_dataset_regr(size=1000):
     X[:, 11] = np.arange(start=0, stop=size, step=1)
     # a lot of missing values
     idx_nan = np.random.choice(size, int(round(size / 2)), replace=False)
-    X[:, 12] = y ** 3 + np.abs(np.random.normal(0, 1, size))
+    X[:, 12] = y**3 + np.abs(np.random.normal(0, 1, size))
     X[idx_nan, 12] = np.nan
 
     # make  it a pandas DF
-    column_names = ['var' + str(i) for i in range(13)]
-    column_names[11] = 'dummy_cat'
+    column_names = ["var" + str(i) for i in range(13)]
+    column_names[11] = "dummy_cat"
     X = pd.DataFrame(X)
     X.columns = column_names
-    X['dummy_cat'] = X['dummy_cat'].astype('category')
-        # low cardinality
-    nice_guys = ['Rick', 'Bender', 'Cartman', 'Morty', 'Fry', 'Vador',
-                 'Thanos', 'Bejita', 'Cell', 'Tinkywinky', 'Lecter',
-                 'Alien', 'Terminator', 'Drago', 'Dracula',
-                 'Krueger', 'Geoffrey', 'Goldfinder', 'Blackbeard',
-                 'Excel', 'SAS', 'Bias', 'Variance', 'Scrum',
-                 'Human', 'Garry', 'Coldplay', 'Imaginedragons',
-                 'Platist', 'Creationist', 'Gruber', 'KeyserSoze', 'Luthor',
-                 'Klaue', 'Bane', 'MarkZ']
-    X['nice_guys'] = np.random.choice(nice_guys, X.shape[0])
+    X["dummy_cat"] = X["dummy_cat"].astype("category")
+    # low cardinality
+    nice_guys = [
+        "Rick",
+        "Bender",
+        "Cartman",
+        "Morty",
+        "Fry",
+        "Vador",
+        "Thanos",
+        "Bejita",
+        "Cell",
+        "Tinkywinky",
+        "Lecter",
+        "Alien",
+        "Terminator",
+        "Drago",
+        "Dracula",
+        "Krueger",
+        "Geoffrey",
+        "Goldfinder",
+        "Blackbeard",
+        "Excel",
+        "SAS",
+        "Bias",
+        "Variance",
+        "Scrum",
+        "Human",
+        "Garry",
+        "Coldplay",
+        "Imaginedragons",
+        "Platist",
+        "Creationist",
+        "Gruber",
+        "KeyserSoze",
+        "Luthor",
+        "Klaue",
+        "Bane",
+        "MarkZ",
+    ]
+    X["nice_guys"] = np.random.choice(nice_guys, X.shape[0])
 
     return X, y, w
 
@@ -798,7 +900,7 @@ def _generated_corr_dataset_classification(size=1000):
     X[:, 0] = z
     X[:, 1] = y * np.abs(np.random.normal(0, 1, size)) + np.random.normal(0, 0.1, size)
     X[:, 2] = -y + np.random.normal(0, 1, size)
-    X[:, 3] = y ** 2 + np.random.normal(0, 1, size)
+    X[:, 3] = y**2 + np.random.normal(0, 1, size)
     X[:, 4] = np.sqrt(y) + np.random.binomial(2, 0.1, size)
 
     # 5 irrelevant features
@@ -813,24 +915,54 @@ def _generated_corr_dataset_classification(size=1000):
     X[:, 11] = np.arange(start=0, stop=size, step=1)
     # a lot of missing values
     idx_nan = np.random.choice(size, int(round(size / 2)), replace=False)
-    X[:, 12] = y ** 3 + np.abs(np.random.normal(0, 1, size))
+    X[:, 12] = y**3 + np.abs(np.random.normal(0, 1, size))
     X[idx_nan, 12] = np.nan
 
     # make  it a pandas DF
-    column_names = ['var' + str(i) for i in range(13)]
-    column_names[11] = 'dummy'
+    column_names = ["var" + str(i) for i in range(13)]
+    column_names[11] = "dummy"
     X = pd.DataFrame(X)
     X.columns = column_names
-    X['dummy'] = X['dummy'].astype('category')
+    X["dummy"] = X["dummy"].astype("category")
 
-    nice_guys = ['Rick', 'Bender', 'Cartman', 'Morty', 'Fry', 'Vador',
-                 'Thanos', 'Bejita', 'Cell', 'Tinkywinky', 'Lecter',
-                 'Alien', 'Terminator', 'Drago', 'Dracula',
-                 'Krueger', 'Geoffrey', 'Goldfinder', 'Blackbeard',
-                 'Excel', 'SAS', 'Bias', 'Variance', 'Scrum',
-                 'Human', 'Garry', 'Coldplay', 'Imaginedragons',
-                 'Platist', 'Creationist', 'Gruber', 'KeyserSoze', 'Luthor',
-                 'Klaue', 'Bane', 'MarkZ']
-    X['nice_guys'] = np.random.choice(nice_guys, X.shape[0])
+    nice_guys = [
+        "Rick",
+        "Bender",
+        "Cartman",
+        "Morty",
+        "Fry",
+        "Vador",
+        "Thanos",
+        "Bejita",
+        "Cell",
+        "Tinkywinky",
+        "Lecter",
+        "Alien",
+        "Terminator",
+        "Drago",
+        "Dracula",
+        "Krueger",
+        "Geoffrey",
+        "Goldfinder",
+        "Blackbeard",
+        "Excel",
+        "SAS",
+        "Bias",
+        "Variance",
+        "Scrum",
+        "Human",
+        "Garry",
+        "Coldplay",
+        "Imaginedragons",
+        "Platist",
+        "Creationist",
+        "Gruber",
+        "KeyserSoze",
+        "Luthor",
+        "Klaue",
+        "Bane",
+        "MarkZ",
+    ]
+    X["nice_guys"] = np.random.choice(nice_guys, X.shape[0])
 
     return X, y, w
